# coding=utf-8
"""Object for calculating PET comfort from DataCollections."""
from __future__ import division

from ..pet import physiologic_equivalent_temperature, pet_category, \
    pet_category_humid, core_temperature_category
from ..parameter.pet import PETParameter
from .base import ComfortCollection
from .solarcal import OutdoorSolarCal

from ladybug._datacollectionbase import BaseCollection

from ladybug.datatype.temperature import Temperature, MeanRadiantTemperature, \
    PhysiologicalEquivalentTemperature, AirTemperature, OperativeTemperature, \
    CoreBodyTemperature, SkinTemperature, ClothingTemperature
from ladybug.datatype.fraction import Fraction, RelativeHumidity
from ladybug.datatype.speed import Speed, AirSpeed
from ladybug.datatype.energyflux import MetabolicRate, EnergyFlux
from ladybug.datatype.pressure import Pressure
from ladybug.datatype.rvalue import ClothingInsulation, RValue
from ladybug.datatype.thermalcondition import CoreTemperatureCategory, \
    ThermalComfort, ThermalCondition, ThermalConditionNinePoint

try:
    from itertools import izip as zip  # python 2
except ImportError:
    pass


class PET(ComfortCollection):
    """PET comfort DataCollection object.

    Args:
        air_temperature: Data Collection of air temperature values in Celsius.
        rel_humidity: Data Collection of relative humidity values in % or a
            single relative humidity value to be used for the whole analysis.
        rad_temperature: Data Collection of mean radiant temperature (MRT)
            values in degrees Celsius or a single MRT value to be used for the whole
            analysis. If None, this will be the same as the air_temperature.
        air_speed: Data Collection of air speed values in m/s or a single
            air_speed value to be used for the whole analysis. If None, this
            will default to 0.1 m/s.
        barometric_pressure: A value or data collection representing atmospheric
            pressure [Pa]. Default is to use air pressure at sea level (101,325 Pa).
        met_rate: Data Collection of metabolic rate in met or a single
            metabolic rate value to be used for the whole analysis. If None,
            default is set to 2.4 met (for walking).
        clo_value: Data Collection of clothing level in clo or a single clothing
            value to be used for the whole analysis. If None, default is
            set to 0.7 clo (for long sleeve shirt and pants).
        body_parameter: Optional PETParameter object to specify the body properties
            of the human subject. The default attempts to model as average of a
            human body as possible.

    Properties:
        * air_temperature
        * rad_temperature
        * air_speed
        * rel_humidity
        * barometric_pressure
        * met_rate
        * clo_value
        * body_parameter
        * physiologic_equivalent_temperature
        * core_body_temperature
        * skin_temperature
        * clothing_temperature
        * operative_temperature
        * is_comfortable
        * thermal_condition
        * pet_category
        * core_temperature_category
        * percent_comfortable
        * percent_uncomfortable
        * percent_neutral
        * percent_hot
        * percent_cold
    """
    _model = 'Physiological Equivalent Temperature'
    __slots__ = (
        '_air_temperature', '_rel_humidity', '_rad_temperature', '_air_speed',
        '_barometric_pressure', '_met_rate', '_clo_value', '_body_par', '_comf_func',
        '_pet', '_t_core', '_t_skin', '_t_clo', '_is_comfortable',
        '_thermal_condition', '_pet_cat', '_core_temp_cat',
        '_air_temperature_coll', '_rel_humidity_coll', '_rad_temperature_coll',
        '_air_speed_coll', '_barometric_pressure_coll', '_met_rate_coll',
        '_clo_value_coll', '_pet_coll', '_t_core_coll', '_t_skin_coll', '_t_clo_coll',
        '_is_comfortable_coll', '_thermal_condition_coll', '_pet_cat_coll',
        '_core_temp_cat_coll', '_to', '_to_coll')

    def __init__(self, air_temperature, rel_humidity,
                 rad_temperature=None, air_speed=None, barometric_pressure=None,
                 met_rate=None, clo_value=None, body_parameter=None):
        """Initialize a PET comfort object from DataCollections of PET inputs.
        """
        # set up the object using air temperature as a base
        self._check_datacoll(air_temperature, Temperature, 'C', 'air_temperature')
        self._input_collections = [air_temperature]
        self._calc_length = len(air_temperature.values)
        self._base_collection = air_temperature

        # check and set required inputs
        self._air_temperature = air_temperature.values
        self._rel_humidity = self._check_input(
            rel_humidity, Fraction, '%', 'rel_humidity')

        # check parameters with defaults
        if rad_temperature is not None:
            self._rad_temperature = self._check_input(
                rad_temperature, Temperature, 'C', 'rad_temperature')
        else:
            self._rad_temperature = self._air_temperature

        if air_speed is not None:
            self._air_speed = self._check_input(
                air_speed, Speed, 'm/s', 'air_speed')
        else:
            self._air_speed = [0.1] * self.calc_length

        if met_rate is not None:
            self._met_rate = self._check_input(
                met_rate, EnergyFlux, 'met', 'met_rate')
        else:
            self._met_rate = [2.4] * self.calc_length

        if clo_value is not None:
            self._clo_value = self._check_input(
                clo_value, RValue, 'clo', 'clo_value')
        else:
            self._clo_value = [0.7] * self.calc_length

        if barometric_pressure is not None:
            self._barometric_pressure = self._check_input(
                barometric_pressure, Pressure, 'Pa', 'barometric_pressure')
        else:
            self._barometric_pressure = [101325.] * self.calc_length

        # check that all input data collections are aligned.
        BaseCollection.are_collections_aligned(self._input_collections)

        # check comfort parameters
        if body_parameter is None:
            self._body_par = PETParameter()
        else:
            assert isinstance(body_parameter, PETParameter), 'body_parameter '\
                'must be a PETParameter object. Got {}'.format(type(body_parameter))
            self._body_par = body_parameter
        self._comf_func = pet_category_humid \
            if self._body_par.humid_acclimated else pet_category

        # calculate PET
        self._calculate_pet()

    @classmethod
    def from_epw(cls, epw, include_wind=True, include_sun=True, met_rate=None,
                 clo_value=None, body_parameter=None):
        """Get a PET comfort object from the conditions within an EPW file.

        Args:
            epw: A ladybug EPW object from which the PET object will be created.
            include_wind: Set to True to include the EPW wind speed in the calculation.
                Setting to False will assume a condition that is shielded from wind
                where the human experiences a very low wind speed of 0.1 m/s. If
                included, the wind speed at ground level will be assumed to be 2/3
                times the meteorological wind speed in the EPW (usually at 10 meters).
                This follows the standard assumed for UTCI. (Default: True to
                include wind).
            include_sun: Set to True to include the mean radiant temperature (MRT) delta
                from both shortwave solar falling directly on people and long wave
                radiant exchange with the sky. Setting to False will assume a shaded
                condition with MRT being equal to the EPW dry bulb temperature. When
                set to True, this calculation will assume no surrounding shade context,
                standing human geometry, and a solar horizontal angle relative to
                front of person (SHARP) of 135 degrees. A SHARP of 135 essentially
                assumes that a person typically faces their side or back to the
                sun to avoid glare. (Default: True to include sun).
            met_rate: Data Collection of metabolic rate in met or a single
                metabolic rate value to be used for the whole analysis. Default: 2.4 met
                (walking at 1 m/s, which is the same assumption used in UTCI).
            clo_value: Data Collection of clothing values rate in clo or a single
                clothing value to be used for the whole analysis. Default: 0.7 clo
                (long sleeve shirt and pants).
            body_parameter: Optional PETParameter object to specify the body properties
                of the human subject. The default attempts to model as average of a
                human body as possible.

        Returns:
            An object with data collections of the PET results as properties.

        Usage:

        .. code-block:: python

            from ladybug.epw import EPW
            from ladybug_comfort.collection.pet import PET

            epw_file_path = './tests/epw/chicago.epw'
            epw = EPW(epw_file_path)
            pet = PET.from_epw(epw, include_wind=True, include_sun=True)

            # 12 values for the average PET in each month
            a = pet.physiologic_equivalent_temperature.average_monthly_per_hour().values
            print(a)
        """
        # get wind input
        if include_wind is True:
            wind_speed = epw.wind_speed.duplicate()
            for i, spd in enumerate(wind_speed):
                wind_speed[i] = spd * (2 / 3)  # 2/3 is the conversion used by UTCI
        else:
            wind_speed = 0.1

        # get the mrt input
        if include_sun is True:
            solarcal_obj = OutdoorSolarCal(epw.location, epw.direct_normal_radiation,
                                           epw.diffuse_horizontal_radiation,
                                           epw.horizontal_infrared_radiation_intensity,
                                           epw.dry_bulb_temperature)
            mrt = solarcal_obj.mean_radiant_temperature
        else:
            mrt = epw.dry_bulb_temperature

        # check the met input
        met_rate = 2.4 if met_rate is None else met_rate

        # return the comfort object
        return cls(epw.dry_bulb_temperature, epw.relative_humidity, mrt, wind_speed,
                   epw.atmospheric_station_pressure, met_rate, clo_value, body_parameter)

    def _calculate_pet(self):
        """Compute PET for each step of the Data Collection."""
        self._setup_list_attributes()
        for ta, tr, vel, rh, met, clo, pr in \
            zip(self._air_temperature, self._rad_temperature,
                self._air_speed, self._rel_humidity,
                self._met_rate, self._clo_value, self._barometric_pressure):
            result = physiologic_equivalent_temperature(
                ta, tr, vel, rh, met, clo, self._body_par.age, self._body_par.sex,
                self._body_par.height, self._body_par.body_mass,
                self._body_par.posture, pr)
            self._append_results_to_lists(result)
            self._assess_comfort(result)

    def _setup_list_attributes(self):
        """Set empty lists for all data collection attributes on this object."""
        self._pet = []
        self._t_core = []
        self._t_skin = []
        self._t_clo = []
        self._to = []
        self._is_comfortable = []
        self._thermal_condition = []
        self._pet_cat = []
        self._core_temp_cat = []

    def _append_results_to_lists(self, result):
        """Append PET results from a dictionary to this object's lists."""
        self._pet.append(result['pet'])
        self._t_core.append(result['t_core'])
        self._t_skin.append(result['t_skin'])
        self._t_clo.append(result['t_clo'])

    def _assess_comfort(self, result):
        """Append determine whether conditions are acceptable from a result dict."""
        pet_cat = self._comf_func(result['pet'])
        t_core_cat = core_temperature_category(result['t_core'])
        comf = pet_cat == 0
        condit = 0
        if pet_cat < 0:
            condit = -1
        elif pet_cat > 0:
            condit = 1
        self._is_comfortable.append(comf)
        self._thermal_condition.append(condit)
        self._pet_cat.append(pet_cat)
        self._core_temp_cat.append(t_core_cat)

    @property
    def air_temperature(self):
        """Data Collection of air temperature values in degrees C."""
        return self._get_coll('_air_temperature_coll', self._air_temperature,
                              AirTemperature, 'C')

    @property
    def rad_temperature(self):
        """Data Collection of mean radiant temperature (MRT) values in degrees C."""
        return self._get_coll('_rad_temperature_coll', self._rad_temperature,
                              MeanRadiantTemperature, 'C')

    @property
    def air_speed(self):
        """Data Collection of air speed values in m/s."""
        return self._get_coll('_air_speed_coll', self._air_speed,
                              AirSpeed, 'm/s')

    @property
    def rel_humidity(self):
        """Data Collection of relative humidity values in %."""
        return self._get_coll('_rel_humidity_coll', self._rel_humidity,
                              RelativeHumidity, '%')

    @property
    def barometric_pressure(self):
        """Data Collection of the barometric pressure in Pa."""
        return self._get_coll('_barometric_pressure_coll', self._barometric_pressure,
                              Pressure, 'Pa')

    @property
    def met_rate(self):
        """Data Collection of metabolic rate in met.

        * 1 met = Metabolic rate of a resting seated person
        * 1.2 met = Metabolic rate of a standing person
        * 2 met = Metabolic rate of a walking person
        """
        return self._get_coll('_met_rate_coll', self._met_rate,
                              MetabolicRate, 'met')

    @property
    def clo_value(self):
        """Data Collection of clothing level of the human subject in clo.

        * 1 clo = Three-piece suit
        * 0.5 clo = Shorts + T-shirt
        * 0 clo = No clothing
        """
        return self._get_coll('_clo_value_coll', self._clo_value,
                              ClothingInsulation, 'clo')

    @property
    def body_parameter(self):
        """PET body parameters that are assigned to this object."""
        return self._body_par

    @property
    def physiologic_equivalent_temperature(self):
        """Data Collection of physiologic equivalent temperature (PET).

        PET is a "feels like" temperature and is defined as the operative temperature
        of a reference environment that would cause the same physiological
        response in the human subject as the environment under study. That is, the
        same skin temperature and core body temperature.
        """
        return self._get_coll('_pet_coll', self._pet,
                              PhysiologicalEquivalentTemperature, 'C')

    @property
    def core_body_temperature(self):
        """Data Collection of core body temperature of the human subject."""
        return self._get_coll('_t_core_coll', self._t_core, CoreBodyTemperature, 'C')

    @property
    def skin_temperature(self):
        """Data Collection of skin temperature of the human subject."""
        return self._get_coll('_t_skin_coll', self._t_skin, SkinTemperature, 'C')

    @property
    def clothing_temperature(self):
        """Data Collection of clothing temperature of the human subject."""
        return self._get_coll('_t_clo_coll', self._t_clo, ClothingTemperature, 'C')

    @property
    def operative_temperature(self):
        """Data Collection of operative temperature in degrees C."""
        if len(self._to) == 0:
            self._to = [(ta + tr) / 2 for ta, tr in
                        zip(self._air_temperature, self._rad_temperature)]
        return self._get_coll('_to_coll', self._to, OperativeTemperature, 'C')

    @property
    def is_comfortable(self):
        """Data Collection of integers noting whether the input conditions are
        acceptable according to the assigned body_parameter.

        Values are one of the following:

        * 0 = uncomfortable
        * 1 = comfortable
        """
        return self._get_coll('_is_comfortable_coll', self._is_comfortable,
                              ThermalComfort, 'condition')

    @property
    def thermal_condition(self):
        """Data Collection of integers noting the thermal status of a subject
        according to the assigned body_parameter.

        Values are one of the following:

        * -1 = cold
        * 0 = netural
        * +1 = hot
        """
        return self._get_coll('_thermal_condition_coll', self._thermal_condition,
                              ThermalCondition, 'condition')

    @property
    def pet_category(self):
        """Data Collection of integers noting the thermal status on a nine-point scale.

        Values are one of the following:

        * -4 = very strong/extreme cold stress
        * -3 = strong cold stress
        * -2 = moderate cold stress
        * -1 = slight cold stress
        * 0 = no thermal stress
        * +1 = slight heat stress
        * +2 = moderate heat stress
        * +3 = strong heat stress
        * +4 = very strong/extreme heat stress
        """
        return self._get_coll('_pet_cat_coll', self._pet_cat,
                              ThermalConditionNinePoint, 'condition')

    @property
    def core_temperature_category(self):
        """Data Collection of integers noting the classification of core body temperature.

        Values are one of the following:

        * -2 = Hypothermia
        * -1 = Cold
        * 0 = Normal
        * 1 = Hot
        * 2 = Hyperthermia
        """
        return self._get_coll('_core_temp_cat_coll', self._core_temp_cat,
                              CoreTemperatureCategory, 'condition')

    @property
    def percent_comfortable(self):
        """The percent of time comfortable given by the assigned body_parameter."""
        return (sum(self._is_comfortable) / self._calc_length) * 100

    @property
    def percent_uncomfortable(self):
        """The percent of time uncomfortable given by the assigned body_parameter."""
        return 100 - self.percent_comfortable

    @property
    def percent_neutral(self):
        """The percent of time that the thermal_condition is neutral."""
        _vals = [1 for x in self._thermal_condition if x == 0]
        return (sum(_vals) / self._calc_length) * 100

    @property
    def percent_cold(self):
        """The percent of time that the thermal_condition is cold."""
        _vals = [1 for x in self._thermal_condition if x == -1]
        return (sum(_vals) / self._calc_length) * 100

    @property
    def percent_hot(self):
        """The percent of time that the thermal_condition is hot."""
        _vals = [1 for x in self._thermal_condition if x == 1]
        return (sum(_vals) / self._calc_length) * 100



# coding=utf-8
"""Object for calculating Adaptive comfort from DataCollections."""
from __future__ import division

from ..adaptive import adaptive_comfort_ashrae55, adaptive_comfort_en15251, \
    adaptive_comfort_conditioned_function, cooling_effect_ashrae55, \
    cooling_effect_en16798, cooling_effect_en15251, t_operative, \
    weighted_running_mean_hourly, weighted_running_mean_daily
from ..parameter.adaptive import AdaptiveParameter
from .base import ComfortCollection

from ladybug._datacollectionbase import BaseCollection
from ladybug.datacollection import HourlyContinuousCollection, DailyCollection, \
    MonthlyCollection, MonthlyPerHourCollection, HourlyDiscontinuousCollection
from ladybug.analysisperiod import AnalysisPeriod

from ladybug.datatype.temperature import Temperature, OperativeTemperature, \
    PrevailingOutdoorTemperature
from ladybug.datatype.speed import Speed, AirSpeed
from ladybug.datatype.thermalcondition import ThermalComfort, ThermalCondition
from ladybug.datatype.temperaturedelta import OperativeTemperatureDelta


class Adaptive(ComfortCollection):
    """Adaptive comfort DataCollection object.

    Args:
        outdoor_temperature: Either one of the following inputs are acceptable:

            * A Data Collection of prevailing outdoor temperature values in C.
              Such a Data Collection must align with the operative_temperature
              input and bear the PrevailingOutdoorTemperature data type in
              its header.
            * A single prevailing outdoor temperature value in C to be used
              for all of the operative_temperature inputs below.
            * A Data Collection of actual outdoor temperatures recorded over
              the entire year. This Data Collection must be continuous and
              must either be an Hourly Collection or Daily Collection. In the event
              that the input comfort_parameter has a prevailing_temperature_method
              of 'Monthly', Monthly collections are also acceptable here. Note
              that, because an annual input is required, this input collection
              does not have to align with the operative_temperature input.

        operative_temperature: Data Collection of operative temperature (To)
            values in degrees Celsius.
        air_speed: Data Collection of air speed values in m/s or a single
            air_speed value to be used for the whole analysis. If None, this
            will default to 0.1 m/s.
        comfort_parameter: Optional AdaptiveParameter object to specify parameters
            under which conditions are considered acceptable. If None, default will
            assume ASHRAE-55 criteria.

    Properties:
        * prevailing_outdoor_temperature
        * operative_temperature
        * air_speed
        * comfort_parameter
        * neutral_temperature
        * degrees_from_neutral
        * is_comfortable
        * thermal_condition
        * cooling_effect
        * percent_comfortable
        * percent_uncomfortable
        * percent_neutral
        * percent_hot
        * percent_cold
    """
    _model = 'Adaptive'
    __slots__ = ('_op_temp', '_air_speed', '_comfort_par', '_t_out', '_prevail_temp',
                 '_neutral_temperature', '_degrees_from_neutral', '_is_comfortable',
                 '_thermal_condition', '_cooling_effect', '_is_comfortable_coll',
                 '_thermal_condition_coll', '_op_temp_coll', '_air_speed_coll',
                 '_comfort_par_coll', '_t_out_coll', '_prevail_temp_coll',
                 '_neutral_temperature_coll', '_degrees_from_neutral_coll',
                 '_cooling_effect_coll')

    def __init__(self, outdoor_temperature, operative_temperature, air_speed=None,
                 comfort_parameter=None): 
        """Initialize an Adaptive comfort object from DataCollections of inputs.
        """
        # set up the object using operative temperature as a base
        self._check_datacoll(operative_temperature, Temperature,
                             'C', 'operative_temperature')
        self._input_collections = [operative_temperature]
        self._calc_length = len(operative_temperature.values)
        self._base_collection = operative_temperature

        # check model inputs
        self._op_temp = operative_temperature.values
        if air_speed is not None:
            self._air_speed = self._check_input(air_speed, Speed, 'm/s', 'air_speed')
        else:
            self._air_speed = [0.1] * self.calc_length

        # check comfort parameters
        if comfort_parameter is None:
            self._comfort_par = AdaptiveParameter()
        else:
            assert isinstance(comfort_parameter, AdaptiveParameter), \
                'comfort_parameter must be an AdaptiveParameter object. '\
                'Got {}'.format(type(comfort_parameter))
            self._comfort_par = comfort_parameter

        # check outdoor_temperature
        self._t_out = outdoor_temperature
        if isinstance(self._t_out, BaseCollection) and not \
                isinstance(self._t_out.header.data_type, PrevailingOutdoorTemperature):
            # it is a data collection of actual recorded outdoor temperatures
            prev_obj = PrevailingTemperature(
                self._t_out, self._comfort_par.avg_month_or_running_mean)
            prevail_collection = prev_obj.get_aligned_prevailing(self._base_collection)
            self._prevail_temp = prevail_collection.values
        else:
            # it is either a data collection or single value of prevailing temperature
            self._prevail_temp = self._check_input(
                self._t_out, PrevailingOutdoorTemperature, 'C', 'outdoor_temperature')

        # calculate Adaptive comfort
        self._calculate_adaptive()

    @classmethod
    def from_air_and_rad_temp(cls, outdoor_temperature, air_temperature,
                              rad_temperature=None, air_speed=None,
                              comfort_parameter=None):
        """Initialize an Adaptive Comfort object from air and radiant temperature."""
        if rad_temperature is None:
            to = air_temperature
        else:
            to = BaseCollection.compute_function_aligned(
                t_operative, [air_temperature, rad_temperature],
                OperativeTemperature(), 'C')
        return cls(outdoor_temperature, to, air_speed, comfort_parameter)

    def _calculate_adaptive(self):
        """Compute Adaptive comfort for each step of the Data Collection.""" 
        # empty properties to be calculated
        self._neutral_temperature = []
        self._degrees_from_neutral = []
        self._is_comfortable = []
        self._thermal_condition = []
        self._cooling_effect = []

        # determine the comfort function to use
        if self._comfort_par.conditioning != 0:
            comf_funct = adaptive_comfort_conditioned_function(
                self._comfort_par.conditioning, self._comfort_par.standard)
        elif self._comfort_par.ashrae_or_en:
            comf_funct = adaptive_comfort_ashrae55
        else:
            comf_funct = adaptive_comfort_en15251

        # determine the cooling effect function to use
        if not self._comfort_par.discrete_or_continuous_air_speed:
            cooling_funct = cooling_effect_en15251
        elif self._comfort_par.ashrae_or_en:
            cooling_funct = cooling_effect_ashrae55
        else:
            cooling_funct = cooling_effect_en16798

        # perform the Adaptive calculation
        for tp, to, vel in zip(self._prevail_temp, self._op_temp, self._air_speed):
            result = comf_funct(tp, to)
            ce = cooling_funct(vel, to, tp)
            comf = self._comfort_par.is_comfortable(result, ce)
            if comf == 0:
                condit = 1 if result['deg_comf'] > 0 else -1
            else:
                condit = 0

            self._neutral_temperature.append(result['t_comf'])
            self._degrees_from_neutral.append(result['deg_comf'])
            self._is_comfortable.append(comf)
            self._thermal_condition.append(condit)
            self._cooling_effect.append(ce)

    @property
    def prevailing_outdoor_temperature(self):
        """Data Collection of prevailing outdoor temperature in degrees C."""
        return self._get_coll('_prevail_temp_coll', self._prevail_temp,
                              PrevailingOutdoorTemperature, 'C')

    @property
    def operative_temperature(self):
        """Data Collection of operative temperature in degrees C."""
        return self._get_coll('_op_temp_coll', self._op_temp, OperativeTemperature, 'C')

    @property
    def air_speed(self):
        """Data Collection of air speed in m/s."""
        return self._get_coll('_air_speed_coll', self._air_speed, AirSpeed, 'm/s')

    @property
    def comfort_parameter(self):
        """Adaptive comfort parameters that are assigned to this object."""
        return self._comfort_par.duplicate()  # duplicate since neutral_offset is setable

    @property
    def neutral_temperature(self):
        """Data Collection of the desired neutral temperature in degrees C."""
        return self._get_coll('_neutral_temperature_coll', self._neutral_temperature,
                              OperativeTemperature, 'C')

    @property
    def degrees_from_neutral(self):
        """Data Collection of the degrees from desired neutral temperature in C."""
        return self._get_coll('_degrees_from_neutral_coll', self._degrees_from_neutral,
                              OperativeTemperatureDelta, 'dC')

    @property
    def is_comfortable(self):
        """Data Collection of integers noting whether the input conditions are
        acceptable according to the assigned comfort_parameter.

        Values are one of the following:
        * 0 = uncomfortable
        * 1 = comfortable
        """
        return self._get_coll('_is_comfortable_coll', self._is_comfortable,
                              ThermalComfort, 'condition')

    @property
    def thermal_condition(self):
        """Data Collection of integers noting the thermal status of a subject
        according to the assigned comfort_parameter.

        Values are one of the following:
        * -1 = cold
        * 0 = netural
        * +1 = hot
        """
        return self._get_coll('_thermal_condition_coll', self._thermal_condition,
                              ThermalCondition, 'condition')

    @property
    def cooling_effect(self):
        """Data Collection of the cooling effect of the air speed in degrees Celsius.

        This is the difference between the air temperature and the
        adjusted air temperature [C].
        """
        return self._get_coll('_cooling_effect_coll', self._cooling_effect,
                              OperativeTemperatureDelta, 'dC')

    @property
    def percent_comfortable(self):
        """The percent of time comfortable given by the assigned comfort_parameter."""
        return (sum(self._is_comfortable) / self._calc_length) * 100

    @property
    def percent_uncomfortable(self):
        """The percent of time uncomfortable given by the assigned comfort_parameter."""
        return 100 - self.percent_comfortable

    @property
    def percent_neutral(self):
        """The percent of time that the thermal_condition is neutral."""
        _vals = [1 for x in self._thermal_condition if x == 0]
        return (sum(_vals) / self._calc_length) * 100

    @property
    def percent_cold(self):
        """The percent of time that the thermal_condition is cold."""
        _vals = [1 for x in self._thermal_condition if x == -1]
        return (sum(_vals) / self._calc_length) * 100

    @property
    def percent_hot(self):
        """The percent of time that the thermal_condition is hot."""
        _vals = [1 for x in self._thermal_condition if x == 1]
        return (sum(_vals) / self._calc_length) * 100


class PrevailingTemperature(object):
    """Get prevailing temperature from annual DataCollections of outdoor temperature.

    Args:
        outdoor_temperature: A Data Collection of outdoor temperatures recorded
            over an entire year. This Data Collection must be continuous and
            must either be an Hourly Collection or Daily Collection. In the event
            that the input avg_month is True (for average monthly
            prevailing method), Monthly collections are also acceptable here.
        avg_month: A boolean to note whether the prevailing outdoor
            temperature is computed from the average monthly temperature (True) or
            a weighted running mean of the last week (False).  The default is True.

    Properties:
        * avg_month_or_running_mean
        * hourly_prevailing_temperature
        * daily_prevailing_temperature
        * monthly_prevailing_temperature
        * monthly_per_hour_prevailing_temperature
    """
    __slots__ = ('_t_out', '_head', '_avg_month', '_hourly_prevail',
                 '_daily_prevail', '_monthly_prevail')

    def __init__(self, outdoor_temperature, avg_month=True):
        """Initialize an prevailing temperature object from DataCollections of inputs.
        """
        # perform checks on the inputs
        acceptabe = (HourlyContinuousCollection, DailyCollection, MonthlyCollection)
        assert isinstance(outdoor_temperature, acceptabe), \
            'outdoor_temperature must be one of the following: {}.\n Got {}'.format(
                acceptabe, type(outdoor_temperature))

        self._t_out = outdoor_temperature.duplicate()
        self._head = self._t_out.header
        self._avg_month = avg_month

        assert isinstance(self._head.data_type, Temperature) and self._head.unit == 'C',\
            'outdoor_temperature must be Temperature in C. Got {} in {}'.format(
                self._head.data_type, self._head.unit)
        assert self._t_out.is_continuous, 'outdoor_temperature must be continuous.'
        assert self._head.analysis_period.is_annual, 'outdoor_temperature must be annual'
        assert isinstance(self._avg_month, bool), 'avg_month' \
            ' must be a boolean. Got {}.'.format(type(self._avg_month))

        # defaults to be calculated
        self._hourly_prevail = []
        self._daily_prevail = []
        self._monthly_prevail = []

        # calculate the base data of prevailing temperature
        if self._avg_month is True:
            if isinstance(self._t_out, (HourlyContinuousCollection, DailyCollection)):
                self._monthly_prevail = self._t_out.average_monthly().values
            elif isinstance(self._t_out, MonthlyCollection):
                self._monthly_prevail = self._t_out.values
        else:
            if isinstance(self._t_out, HourlyContinuousCollection):
                self._hourly_prevail = weighted_running_mean_hourly(self._t_out.values)
            elif isinstance(self._t_out, DailyCollection):
                self._daily_prevail = weighted_running_mean_daily(self._t_out.values)
                for val in self._daily_prevail:
                    self._hourly_prevail.extend([val] * 24)
            else:
                raise TypeError('outdoor_temperature must be hourly or daily when '
                                'avg_month is False.')

    @property
    def avg_month(self):
        """The input avg_month."""
        return self._avg_month

    @property
    def hourly_prevailing_temperature(self):
        """HourlyContinuousCollection of prevailing outdoor temperature in C."""
        if self._hourly_prevail == []:
            self._hourly_prevail_from_monthly()
        return HourlyContinuousCollection(self._get_header(), self._hourly_prevail)

    @property
    def daily_prevailing_temperature(self):
        """DailyCollection of prevailing outdoor temperature in C."""
        if self._daily_prevail == []:
            if self._avg_month is True:
                self._daily_prevail_from_monthly()
            else:
                self._daily_prevail_from_hourly()
        return DailyCollection(self._get_header(), self._daily_prevail,
                               self._head.analysis_period.doys_int)

    @property
    def monthly_prevailing_temperature(self):
        """MonthlyCollection of prevailing outdoor temperature in C."""
        if self._monthly_prevail == []:
            return self.hourly_prevailing_temperature.average_monthly()
        else:
            return MonthlyCollection(self._get_header(), self._monthly_prevail,
                                     self._head.analysis_period.months_int)

    @property
    def monthly_per_hour_prevailing_temperature(self):
        """MonthlyPerHourCollection of prevailing outdoor temperature in C."""
        mon_per_hr_vals = []
        for val in self.monthly_prevailing_temperature:
            mon_per_hr_vals.extend([val] * 24)
        return MonthlyPerHourCollection(self._get_header(), mon_per_hr_vals,
                                        self._head.analysis_period.months_per_hour)

    def hourly_prevailing_temperature_timestep(self, timestep):
        """HourlyContinuousCollection of prevailing temperature at timestep."""
        if timestep != 1:
            hourly_coll = self.hourly_prevailing_temperature
            _new_values = []
            for val in hourly_coll.values:
                _new_values.extend([val] * timestep)
            a_per = hourly_coll.header.analysis_period
            _new_a_per = AnalysisPeriod(a_per.st_month, a_per.st_day, a_per.st_hour,
                                        a_per.end_month, a_per.end_day, a_per.end_hour,
                                        timestep, a_per.is_leap_year)
            _new_header = hourly_coll.header.duplicate()
            _new_header._analysis_period = _new_a_per
            return HourlyContinuousCollection(_new_header, _new_values)
        return self.hourly_prevailing_temperature

    def get_aligned_prevailing(self, collection):
        """"Get a Prevailing Temperature Collection aligned with input collection."""
        if isinstance(collection, HourlyContinuousCollection):
            new_coll = self.hourly_prevailing_temperature_timestep(
                collection.header.analysis_period.timestep)
            if not collection.header.analysis_period.is_annual:
                new_coll = new_coll.filter_by_analysis_period(
                    collection.header.analysis_period)
            return new_coll

        new_coll = collection.get_aligned_collection(
            data_type=PrevailingOutdoorTemperature(), unit='C')
        if isinstance(collection, HourlyDiscontinuousCollection):
            prevail_val_dict = self.hourly_prevailing_temperature_timestep(
                collection.header.analysis_period.timestep).moys_dict
            for i, datetime in enumerate(new_coll.datetimes):
                new_coll[i] = prevail_val_dict[datetime.moy]
        elif isinstance(collection, DailyCollection):
            daily_temps = self.daily_prevailing_temperature
            for i, datetime in enumerate(new_coll.datetimes):
                new_coll[i] = daily_temps[datetime - 1]
        elif isinstance(collection, MonthlyCollection):
            monthly_temps = self.monthly_prevailing_temperature
            for i, datetime in enumerate(new_coll.datetimes):
                new_coll[i] = monthly_temps[datetime - 1]
        elif isinstance(collection, MonthlyPerHourCollection):
            mon_per_hr = self.monthly_per_hour_prevailing_temperature
            monthly_per_hour_dict = {}
            for val, dt in zip(mon_per_hr.values, mon_per_hr.datetimes):
                monthly_per_hour_dict[dt] = val
            for i, datetime in enumerate(new_coll.datetimes):
                new_coll[i] = monthly_per_hour_dict[datetime]

        return new_coll

    def _hourly_prevail_from_monthly(self):
        for i, days in enumerate(self._head.analysis_period._num_of_days_each_month):
            self._hourly_prevail.extend([self._monthly_prevail[i]] * days * 24)

    def _daily_prevail_from_monthly(self):
        for i, days in enumerate(self._head.analysis_period._num_of_days_each_month):
            self._daily_prevail.extend([self._monthly_prevail[i]] * days)

    def _daily_prevail_from_hourly(self):
        for i in range(0, len(self._hourly_prevail), 24):
            self._daily_prevail.append(self._hourly_prevail[i])

    def _get_header(self):
        new_header = self._head.duplicate()
        new_header._data_type = PrevailingOutdoorTemperature()
        return new_header
